package javax.swing.plaf.basic;

import java.awt.Insets;
import java.beans.PropertyChangeEvent;
import java.awt.event.MouseEvent;
import java.awt.event.FocusEvent;
import java.awt.event.ContainerEvent;
import java.awt.event.ActionEvent;
import sun.swing.UIAction;
import java.awt.Graphics;
import java.awt.event.WindowListener;
import java.awt.Dimension;
import java.awt.IllegalComponentStateException;
import java.awt.LayoutManager;
import java.awt.BorderLayout;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.awt.Point;
import javax.swing.JToggleButton;
import java.awt.Dialog;
import java.awt.Frame;
import javax.swing.JDialog;
import java.awt.Window;
import javax.swing.JRootPane;
import java.awt.GraphicsConfiguration;
import javax.swing.UIDefaults;
import javax.swing.border.CompoundBorder;
import sun.swing.DefaultLookup;
import javax.swing.InputMap;
import javax.swing.SwingUtilities;
import java.awt.Component;
import javax.swing.UIManager;
import javax.swing.plaf.UIResource;
import javax.swing.LookAndFeel;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.KeyStroke;
import java.util.Hashtable;
import javax.swing.AbstractButton;
import java.util.HashMap;
import javax.swing.border.Border;
import java.awt.event.FocusListener;
import java.awt.event.ContainerListener;
import java.beans.PropertyChangeListener;
import javax.swing.event.MouseInputListener;
import java.awt.Color;
import java.awt.Container;
import javax.swing.RootPaneContainer;
import javax.swing.JFrame;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.plaf.ToolBarUI;

public class BasicToolBarUI extends ToolBarUI implements SwingConstants
{
    protected JToolBar toolBar;
    private boolean floating;
    private int floatingX;
    private int floatingY;
    private JFrame floatingFrame;
    private RootPaneContainer floatingToolBar;
    protected BasicToolBarUI.DragWindow dragWindow;
    private Container dockingSource;
    private int dockingSensitivity;
    protected int focusedCompIndex;
    protected Color dockingColor;
    protected Color floatingColor;
    protected Color dockingBorderColor;
    protected Color floatingBorderColor;
    protected MouseInputListener dockingListener;
    protected PropertyChangeListener propertyListener;
    protected ContainerListener toolBarContListener;
    protected FocusListener toolBarFocusListener;
    private BasicToolBarUI.Handler handler;
    protected String constraintBeforeFloating;
    private static String IS_ROLLOVER;
    private static Border rolloverBorder;
    private static Border nonRolloverBorder;
    private static Border nonRolloverToggleBorder;
    private boolean rolloverBorders;
    private HashMap<AbstractButton, Border> borderTable;
    private Hashtable<AbstractButton, Boolean> rolloverTable;
    @Deprecated
    protected KeyStroke upKey;
    @Deprecated
    protected KeyStroke downKey;
    @Deprecated
    protected KeyStroke leftKey;
    @Deprecated
    protected KeyStroke rightKey;
    private static String FOCUSED_COMP_INDEX;
    
    public BasicToolBarUI() {
        this.dockingSensitivity = 0;
        this.focusedCompIndex = -1;
        this.dockingColor = null;
        this.floatingColor = null;
        this.dockingBorderColor = null;
        this.floatingBorderColor = null;
        this.constraintBeforeFloating = "North";
        this.rolloverBorders = false;
        this.borderTable = new HashMap();
        this.rolloverTable = new Hashtable();
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new BasicToolBarUI();
    }
    
    @Override
    public void installUI(JComponent c) {
        this.toolBar = (JToolBar)c;
        this.installDefaults();
        this.installComponents();
        this.installListeners();
        this.installKeyboardActions();
        this.dockingSensitivity = 0;
        this.floating = false;
        int n = 0;
        this.floatingY = n;
        this.floatingX = n;
        this.floatingToolBar = null;
        this.setOrientation(this.toolBar.getOrientation());
        LookAndFeel.installProperty(c, "opaque", Boolean.TRUE);
        if (c.getClientProperty(BasicToolBarUI.FOCUSED_COMP_INDEX) != null) {
            this.focusedCompIndex = (int)(Integer)c.getClientProperty(BasicToolBarUI.FOCUSED_COMP_INDEX);
        }
    }
    
    @Override
    public void uninstallUI(JComponent component) {
        this.uninstallDefaults();
        this.uninstallComponents();
        this.uninstallListeners();
        this.uninstallKeyboardActions();
        if (this.isFloating()) {
            this.setFloating(false, null);
        }
        this.floatingToolBar = null;
        this.dragWindow = null;
        this.dockingSource = null;
        component.putClientProperty(BasicToolBarUI.FOCUSED_COMP_INDEX, Integer.valueOf(this.focusedCompIndex));
    }
    
    protected void installDefaults() {
        LookAndFeel.installBorder(this.toolBar, "ToolBar.border");
        LookAndFeel.installColorsAndFont(this.toolBar, "ToolBar.background", "ToolBar.foreground", "ToolBar.font");
        if (this.dockingColor == null || this.dockingColor instanceof UIResource) {
            this.dockingColor = UIManager.getColor("ToolBar.dockingBackground");
        }
        if (this.floatingColor == null || this.floatingColor instanceof UIResource) {
            this.floatingColor = UIManager.getColor("ToolBar.floatingBackground");
        }
        if (this.dockingBorderColor == null || this.dockingBorderColor instanceof UIResource) {
            this.dockingBorderColor = UIManager.getColor("ToolBar.dockingForeground");
        }
        if (this.floatingBorderColor == null || this.floatingBorderColor instanceof UIResource) {
            this.floatingBorderColor = UIManager.getColor("ToolBar.floatingForeground");
        }
        Object o = this.toolBar.getClientProperty(BasicToolBarUI.IS_ROLLOVER);
        if (o == null) {
            o = UIManager.get("ToolBar.isRollover");
        }
        if (o != null) {
            this.rolloverBorders = (boolean)o;
        }
        if (BasicToolBarUI.rolloverBorder == null) {
            BasicToolBarUI.rolloverBorder = this.createRolloverBorder();
        }
        if (BasicToolBarUI.nonRolloverBorder == null) {
            BasicToolBarUI.nonRolloverBorder = this.createNonRolloverBorder();
        }
        if (BasicToolBarUI.nonRolloverToggleBorder == null) {
            BasicToolBarUI.nonRolloverToggleBorder = this.createNonRolloverToggleBorder();
        }
        this.setRolloverBorders(this.isRolloverBorders());
    }
    
    protected void uninstallDefaults() {
        LookAndFeel.uninstallBorder(this.toolBar);
        this.dockingColor = null;
        this.floatingColor = null;
        this.dockingBorderColor = null;
        this.floatingBorderColor = null;
        this.installNormalBorders(this.toolBar);
        BasicToolBarUI.rolloverBorder = null;
        BasicToolBarUI.nonRolloverBorder = null;
        BasicToolBarUI.nonRolloverToggleBorder = null;
    }
    
    protected void installComponents() {
    }
    
    protected void uninstallComponents() {
    }
    
    protected void installListeners() {
        this.dockingListener = this.createDockingListener();
        if (this.dockingListener != null) {
            this.toolBar.addMouseMotionListener(this.dockingListener);
            this.toolBar.addMouseListener(this.dockingListener);
        }
        this.propertyListener = this.createPropertyListener();
        if (this.propertyListener != null) {
            this.toolBar.addPropertyChangeListener(this.propertyListener);
        }
        this.toolBarContListener = this.createToolBarContListener();
        if (this.toolBarContListener != null) {
            this.toolBar.addContainerListener(this.toolBarContListener);
        }
        this.toolBarFocusListener = this.createToolBarFocusListener();
        if (this.toolBarFocusListener != null) {
            Component[] components = this.toolBar.getComponents();
            for (int length = components.length, i = 0; i < length; ++i) {
                components[i].addFocusListener(this.toolBarFocusListener);
            }
        }
    }
    
    protected void uninstallListeners() {
        if (this.dockingListener != null) {
            this.toolBar.removeMouseMotionListener(this.dockingListener);
            this.toolBar.removeMouseListener(this.dockingListener);
            this.dockingListener = null;
        }
        if (this.propertyListener != null) {
            this.toolBar.removePropertyChangeListener(this.propertyListener);
            this.propertyListener = null;
        }
        if (this.toolBarContListener != null) {
            this.toolBar.removeContainerListener(this.toolBarContListener);
            this.toolBarContListener = null;
        }
        if (this.toolBarFocusListener != null) {
            Component[] components = this.toolBar.getComponents();
            for (int length = components.length, i = 0; i < length; ++i) {
                components[i].removeFocusListener(this.toolBarFocusListener);
            }
            this.toolBarFocusListener = null;
        }
        this.handler = null;
    }
    
    protected void installKeyboardActions() {
        SwingUtilities.replaceUIInputMap(this.toolBar, 1, this.getInputMap(1));
        LazyActionMap.installLazyActionMap(this.toolBar, BasicToolBarUI.class, "ToolBar.actionMap");
    }
    
    InputMap getInputMap(int n) {
        if (n == 1) {
            return (InputMap)DefaultLookup.get(this.toolBar, this, "ToolBar.ancestorInputMap");
        }
        return null;
    }
    
    static void loadActionMap(LazyActionMap lazyActionMap) {
        lazyActionMap.put(new BasicToolBarUI.Actions("navigateRight"));
        lazyActionMap.put(new BasicToolBarUI.Actions("navigateLeft"));
        lazyActionMap.put(new BasicToolBarUI.Actions("navigateUp"));
        lazyActionMap.put(new BasicToolBarUI.Actions("navigateDown"));
    }
    
    protected void uninstallKeyboardActions() {
        SwingUtilities.replaceUIActionMap(this.toolBar, null);
        SwingUtilities.replaceUIInputMap(this.toolBar, 1, null);
    }
    
    protected void navigateFocusedComp(int n) {
        int componentCount = this.toolBar.getComponentCount();
        switch (n) {
            case 3:
            case 5: {
                if (this.focusedCompIndex >= 0) {
                    if (this.focusedCompIndex >= componentCount) {
                        break;
                    }
                    int i = this.focusedCompIndex + 1;
                    while (i != this.focusedCompIndex) {
                        if (i >= componentCount) {
                            i = 0;
                        }
                        Component componentAtIndex = this.toolBar.getComponentAtIndex(i++);
                        if (componentAtIndex != null && componentAtIndex.isFocusTraversable() && componentAtIndex.isEnabled()) {
                            componentAtIndex.requestFocus();
                            break;
                        }
                        continue;
                    }
                    break;
                }
                else {
                    break;
                }
            }
            case 1:
            case 7: {
                if (this.focusedCompIndex >= 0) {
                    if (this.focusedCompIndex >= componentCount) {
                        break;
                    }
                    int j = this.focusedCompIndex - 1;
                    while (j != this.focusedCompIndex) {
                        if (j < 0) {
                            j = componentCount - 1;
                        }
                        Component componentAtIndex2 = this.toolBar.getComponentAtIndex(j--);
                        if (componentAtIndex2 != null && componentAtIndex2.isFocusTraversable() && componentAtIndex2.isEnabled()) {
                            componentAtIndex2.requestFocus();
                            break;
                        }
                        continue;
                    }
                    break;
                }
                else {
                    break;
                }
            }
        }
    }
    
    protected Border createRolloverBorder() {
        Object value = UIManager.get("ToolBar.rolloverBorder");
        if (value != null) {
            return (Border)value;
        }
        UIDefaults lookAndFeelDefaults = UIManager.getLookAndFeelDefaults();
        return new CompoundBorder(new BasicBorders.RolloverButtonBorder(lookAndFeelDefaults.getColor("controlShadow"), lookAndFeelDefaults.getColor("controlDkShadow"), lookAndFeelDefaults.getColor("controlHighlight"), lookAndFeelDefaults.getColor("controlLtHighlight")), new BasicBorders.RolloverMarginBorder());
    }
    
    protected Border createNonRolloverBorder() {
        Object value = UIManager.get("ToolBar.nonrolloverBorder");
        if (value != null) {
            return (Border)value;
        }
        UIDefaults lookAndFeelDefaults = UIManager.getLookAndFeelDefaults();
        return new CompoundBorder(new BasicBorders.ButtonBorder(lookAndFeelDefaults.getColor("Button.shadow"), lookAndFeelDefaults.getColor("Button.darkShadow"), lookAndFeelDefaults.getColor("Button.light"), lookAndFeelDefaults.getColor("Button.highlight")), new BasicBorders.RolloverMarginBorder());
    }
    
    private Border createNonRolloverToggleBorder() {
        UIDefaults lookAndFeelDefaults = UIManager.getLookAndFeelDefaults();
        return new CompoundBorder(new BasicBorders.RadioButtonBorder(lookAndFeelDefaults.getColor("ToggleButton.shadow"), lookAndFeelDefaults.getColor("ToggleButton.darkShadow"), lookAndFeelDefaults.getColor("ToggleButton.light"), lookAndFeelDefaults.getColor("ToggleButton.highlight")), new BasicBorders.RolloverMarginBorder());
    }
    
    protected JFrame createFloatingFrame(JToolBar c) {
        Window windowAncestor = SwingUtilities.getWindowAncestor(c);
        BasicToolBarUI.1 o = new JFrame(c.getName(), (windowAncestor != null) ? windowAncestor.getGraphicsConfiguration() : null) {
            @Override
            protected JRootPane createRootPane() {
                JRootPane rootPane = new JRootPane() {
                    private boolean packing = false;
                    
                    @Override
                    public void validate() {
                        super.validate();
                        if (!this.packing) {
                            this.packing = true;
                            BasicToolBarUI.1.this.pack();
                            this.packing = false;
                        }
                    }
                };
                rootPane.setOpaque(true);
                return rootPane;
            }
        };
        o.getRootPane().setName("ToolBar.FloatingFrame");
        o.setResizable(false);
        o.addWindowListener(this.createFrameListener());
        return o;
    }
    
    protected RootPaneContainer createFloatingWindow(JToolBar c) {
        Window windowAncestor = SwingUtilities.getWindowAncestor(c);
        class ToolBarDialog extends JDialog
        {
            public ToolBarDialog(String title, boolean modal) {
                super(owner, title, modal);
            }
            
            public ToolBarDialog(String title, boolean modal) {
                super(owner, title, modal);
            }
            
            @Override
            protected JRootPane createRootPane() {
                JRootPane rootPane = new JRootPane() {
                    private boolean packing = false;
                    
                    @Override
                    public void validate() {
                        super.validate();
                        if (!this.packing) {
                            this.packing = true;
                            BasicToolBarUI.1ToolBarDialog.this.pack();
                            this.packing = false;
                        }
                    }
                };
                rootPane.setOpaque(true);
                return rootPane;
            }
        }
        BasicToolBarUI.1ToolBarDialog toolBarDialog;
        if ((windowAncestor instanceof Frame)) {
            toolBarDialog = new ToolBarDialog(c.getName(), false);
        }
        else if ((windowAncestor instanceof Dialog)) {
            toolBarDialog = new ToolBarDialog(c.getName(), false);
        }
        else {
            toolBarDialog = new ToolBarDialog(c.getName(), false);
        }
        toolBarDialog.getRootPane().setName("ToolBar.FloatingWindow");
        toolBarDialog.setTitle(c.getName());
        toolBarDialog.setResizable(false);
        toolBarDialog.addWindowListener(this.createFrameListener());
        return toolBarDialog;
    }
    
    protected BasicToolBarUI.DragWindow createDragWindow(JToolBar toolBar) {
        Window window = null;
        if (this.toolBar != null) {
            Container container;
            for (container = this.toolBar.getParent(); container != null && !(container instanceof Window); container = container.getParent()) {}
            if (container != null && container instanceof Window) {
                window = (Window)container;
            }
        }
        if (this.floatingToolBar == null) {
            this.floatingToolBar = this.createFloatingWindow(this.toolBar);
        }
        if ((this.floatingToolBar instanceof Window)) {
            window = (Window)this.floatingToolBar;
        }
        return new BasicToolBarUI.DragWindow(window);
    }
    
    public boolean isRolloverBorders() {
        return this.rolloverBorders;
    }
    
    public void setRolloverBorders(boolean rolloverBorders) {
        this.rolloverBorders = rolloverBorders;
        if (this.rolloverBorders) {
            this.installRolloverBorders(this.toolBar);
        }
        else {
            this.installNonRolloverBorders(this.toolBar);
        }
    }
    
    protected void installRolloverBorders(JComponent component) {
        Component[] components;
        for (Component borderToRollover : components = component.getComponents()) {
            if ((borderToRollover instanceof JComponent)) {
                ((JComponent)borderToRollover).updateUI();
                this.setBorderToRollover(borderToRollover);
            }
        }
    }
    
    protected void installNonRolloverBorders(JComponent component) {
        Component[] components;
        for (Component borderToNonRollover : components = component.getComponents()) {
            if ((borderToNonRollover instanceof JComponent)) {
                ((JComponent)borderToNonRollover).updateUI();
                this.setBorderToNonRollover(borderToNonRollover);
            }
        }
    }
    
    protected void installNormalBorders(JComponent component) {
        Component[] components = component.getComponents();
        for (int length = components.length, i = 0; i < length; ++i) {
            this.setBorderToNormal(components[i]);
        }
    }
    
    protected void setBorderToRollover(Component component) {
        if ((component instanceof AbstractButton)) {
            AbstractButton key = (AbstractButton)component;
            Border border = (Border)this.borderTable.get(key);
            if (border == null || border instanceof UIResource) {
                this.borderTable.put(key, key.getBorder());
            }
            if ((key.getBorder() instanceof UIResource)) {
                key.setBorder(this.getRolloverBorder(key));
            }
            this.rolloverTable.put(key, key.isRolloverEnabled() ? Boolean.TRUE : Boolean.FALSE);
            key.setRolloverEnabled(true);
        }
    }
    
    protected Border getRolloverBorder(AbstractButton abstractButton) {
        return BasicToolBarUI.rolloverBorder;
    }
    
    protected void setBorderToNonRollover(Component component) {
        if ((component instanceof AbstractButton)) {
            AbstractButton key = (AbstractButton)component;
            Border border = (Border)this.borderTable.get(key);
            if (border == null || border instanceof UIResource) {
                this.borderTable.put(key, key.getBorder());
            }
            if ((key.getBorder() instanceof UIResource)) {
                key.setBorder(this.getNonRolloverBorder(key));
            }
            this.rolloverTable.put(key, key.isRolloverEnabled() ? Boolean.TRUE : Boolean.FALSE);
            key.setRolloverEnabled(false);
        }
    }
    
    protected Border getNonRolloverBorder(AbstractButton abstractButton) {
        if ((abstractButton instanceof JToggleButton)) {
            return BasicToolBarUI.nonRolloverToggleBorder;
        }
        return BasicToolBarUI.nonRolloverBorder;
    }
    
    protected void setBorderToNormal(Component component) {
        if ((component instanceof AbstractButton)) {
            AbstractButton abstractButton = (AbstractButton)component;
            abstractButton.setBorder((Border)this.borderTable.remove(abstractButton));
            Boolean b = (Boolean)this.rolloverTable.remove(abstractButton);
            if (b != null) {
                abstractButton.setRolloverEnabled((boolean)b);
            }
        }
    }
    
    public void setFloatingLocation(int floatingX, int floatingY) {
        this.floatingX = floatingX;
        this.floatingY = floatingY;
    }
    
    public boolean isFloating() {
        return this.floating;
    }
    
    public void setFloating(boolean floating, Point point) {
        if (this.toolBar.isFloatable()) {
            boolean visible = false;
            Window windowAncestor = SwingUtilities.getWindowAncestor(this.toolBar);
            if (windowAncestor != null) {
                visible = windowAncestor.isVisible();
            }
            if (this.dragWindow != null) {
                this.dragWindow.setVisible(false);
            }
            this.floating = floating;
            if (this.floatingToolBar == null) {
                this.floatingToolBar = this.createFloatingWindow(this.toolBar);
            }
            if (floating) {
                if (this.dockingSource == null) {
                    (this.dockingSource = this.toolBar.getParent()).remove(this.toolBar);
                }
                this.constraintBeforeFloating = this.calculateConstraint();
                if (this.propertyListener != null) {
                    UIManager.addPropertyChangeListener(this.propertyListener);
                }
                this.floatingToolBar.getContentPane().add(this.toolBar, "Center");
                if ((this.floatingToolBar instanceof Window)) {
                    ((Window)this.floatingToolBar).pack();
                    ((Window)this.floatingToolBar).setLocation(this.floatingX, this.floatingY);
                    if (!(visible == 0)) {
                        ((Window)this.floatingToolBar).show();
                    }
                    else {
                        windowAncestor.addWindowListener(new WindowAdapter() {
                            @Override
                            public void windowOpened(WindowEvent windowEvent) {
                                ((Window)BasicToolBarUI.this.floatingToolBar).show();
                            }
                        });
                    }
                }
            }
            else {
                if (this.floatingToolBar == null) {
                    this.floatingToolBar = this.createFloatingWindow(this.toolBar);
                }
                if ((this.floatingToolBar instanceof Window)) {
                    ((Window)this.floatingToolBar).setVisible(false);
                }
                this.floatingToolBar.getContentPane().remove(this.toolBar);
                String dockingConstraint = this.getDockingConstraint(this.dockingSource, point);
                if (dockingConstraint == null) {
                    dockingConstraint = "North";
                }
                this.setOrientation(this.mapConstraintToOrientation(dockingConstraint));
                if (this.dockingSource == null) {
                    this.dockingSource = this.toolBar.getParent();
                }
                if (this.propertyListener != null) {
                    UIManager.removePropertyChangeListener(this.propertyListener);
                }
                this.dockingSource.add(dockingConstraint, this.toolBar);
            }
            this.dockingSource.invalidate();
            Container parent = this.dockingSource.getParent();
            if (parent != null) {
                parent.validate();
            }
            this.dockingSource.repaint();
        }
    }
    
    private int mapConstraintToOrientation(String s) {
        int orientation = this.toolBar.getOrientation();
        if (s != null) {
            if (s.equals("East") || s.equals("West")) {
                orientation = 1;
            }
            else if (s.equals("North") || s.equals("South")) {
                orientation = 0;
            }
        }
        return orientation;
    }
    
    public void setOrientation(int n) {
        this.toolBar.setOrientation(n);
        if (this.dragWindow != null) {
            this.dragWindow.setOrientation(n);
        }
    }
    
    public Color getDockingColor() {
        return this.dockingColor;
    }
    
    public void setDockingColor(Color dockingColor) {
        this.dockingColor = dockingColor;
    }
    
    public Color getFloatingColor() {
        return this.floatingColor;
    }
    
    public void setFloatingColor(Color floatingColor) {
        this.floatingColor = floatingColor;
    }
    
    private boolean isBlocked(Component component, Object constraints) {
        if ((component instanceof Container)) {
            Container target = (Container)component;
            LayoutManager layout = target.getLayout();
            if ((layout instanceof BorderLayout)) {
                Component layoutComponent = ((BorderLayout)layout).getLayoutComponent(target, constraints);
                return layoutComponent != null && layoutComponent != this.toolBar;
            }
        }
        return false;
    }
    
    public boolean canDock(Component component, Point point) {
        return point != null && this.getDockingConstraint(component, point) != null;
    }
    
    private String calculateConstraint() {
        String s = null;
        LayoutManager layout = this.dockingSource.getLayout();
        if ((layout instanceof BorderLayout)) {
            s = (String)((BorderLayout)layout).getConstraints(this.toolBar);
        }
        return (s != null) ? s : this.constraintBeforeFloating;
    }
    
    private String getDockingConstraint(Component component, Point p2) {
        if (p2 == null) {
            return this.constraintBeforeFloating;
        }
        if (component.contains(p2)) {
            this.dockingSensitivity = ((this.toolBar.getOrientation() == 0) ? this.toolBar.getSize().height : this.toolBar.getSize().width);
            if (p2.y < this.dockingSensitivity && !this.isBlocked(component, "North")) {
                return "North";
            }
            if (p2.x >= component.getWidth() - this.dockingSensitivity && !this.isBlocked(component, "East")) {
                return "East";
            }
            if (p2.x < this.dockingSensitivity && !this.isBlocked(component, "West")) {
                return "West";
            }
            if (p2.y >= component.getHeight() - this.dockingSensitivity && !this.isBlocked(component, "South")) {
                return "South";
            }
        }
        return null;
    }
    
    protected void dragTo(Point point, Point point2) {
        if (this.toolBar.isFloatable()) {
            try {
                if (this.dragWindow == null) {
                    this.dragWindow = this.createDragWindow(this.toolBar);
                }
                Point offset = this.dragWindow.getOffset();
                if (offset == null) {
                    Dimension preferredSize = this.toolBar.getPreferredSize();
                    offset = new Point(preferredSize.width / 2, preferredSize.height / 2);
                    this.dragWindow.setOffset(offset);
                }
                Point o = new Point(point2.x + point.x, point2.y + point.y);
                Point o2 = new Point(o.x - offset.x, o.y - offset.y);
                if (this.dockingSource == null) {
                    this.dockingSource = this.toolBar.getParent();
                }
                this.constraintBeforeFloating = this.calculateConstraint();
                Point locationOnScreen = this.dockingSource.getLocationOnScreen();
                Point o3 = new Point(o.x - locationOnScreen.x, o.y - locationOnScreen.y);
                if (this.canDock(this.dockingSource, o3)) {
                    this.dragWindow.setBackground(this.getDockingColor());
                    this.dragWindow.setOrientation(this.mapConstraintToOrientation(this.getDockingConstraint(this.dockingSource, o3)));
                    this.dragWindow.setBorderColor(this.dockingBorderColor);
                }
                else {
                    this.dragWindow.setBackground(this.getFloatingColor());
                    this.dragWindow.setBorderColor(this.floatingBorderColor);
                    this.dragWindow.setOrientation(this.toolBar.getOrientation());
                }
                this.dragWindow.setLocation(o2.x, o2.y);
                if (!this.dragWindow.isVisible()) {
                    Dimension preferredSize2 = this.toolBar.getPreferredSize();
                    this.dragWindow.setSize(preferredSize2.width, preferredSize2.height);
                    this.dragWindow.show();
                }
            }
            catch (IllegalComponentStateException ex) {}
        }
    }
    
    protected void floatAt(Point point, Point point2) {
        if (this.toolBar.isFloatable()) {
            try {
                Point offset = this.dragWindow.getOffset();
                if (offset == null) {
                    offset = point;
                    this.dragWindow.setOffset(offset);
                }
                Point o = new Point(point2.x + point.x, point2.y + point.y);
                this.setFloatingLocation(o.x - offset.x, o.y - offset.y);
                if (this.dockingSource != null) {
                    Point locationOnScreen = this.dockingSource.getLocationOnScreen();
                    Point o2 = new Point(o.x - locationOnScreen.x, o.y - locationOnScreen.y);
                    if (this.canDock(this.dockingSource, o2)) {
                        this.setFloating(false, o2);
                    }
                    else {
                        this.setFloating(true, null);
                    }
                }
                else {
                    this.setFloating(true, null);
                }
                this.dragWindow.setOffset(null);
            }
            catch (IllegalComponentStateException ex) {}
        }
    }
    
    private BasicToolBarUI.Handler getHandler() {
        if (this.handler == null) {
            this.handler = new BasicToolBarUI.Handler();
        }
        return this.handler;
    }
    
    protected ContainerListener createToolBarContListener() {
        return this.getHandler();
    }
    
    protected FocusListener createToolBarFocusListener() {
        return this.getHandler();
    }
    
    protected PropertyChangeListener createPropertyListener() {
        return this.getHandler();
    }
    
    protected MouseInputListener createDockingListener() {
        this.getHandler().tb = this.toolBar;
        return this.getHandler();
    }
    
    protected WindowListener createFrameListener() {
        return new BasicToolBarUI.FrameListener();
    }
    
    protected void paintDragWindow(Graphics graphics) {
        graphics.setColor(this.dragWindow.getBackground());
        int width = this.dragWindow.getWidth();
        int height = this.dragWindow.getHeight();
        graphics.fillRect(0, 0, width, height);
        graphics.setColor(this.dragWindow.getBorderColor());
        graphics.drawRect(0, 0, width - 1, height - 1);
    }
    
    static BasicToolBarUI.Handler access$500(BasicToolBarUI basicToolBarUI) {
        return basicToolBarUI.getHandler();
    }
    
    static {
        BasicToolBarUI.IS_ROLLOVER = "JToolBar.isRollover";
        BasicToolBarUI.FOCUSED_COMP_INDEX = "JToolBar.focusedCompIndex";
    }
    
    private static class Actions extends UIAction
    {
        private static final String NAVIGATE_RIGHT = "navigateRight";
        private static final String NAVIGATE_LEFT = "navigateLeft";
        private static final String NAVIGATE_UP = "navigateUp";
        private static final String NAVIGATE_DOWN = "navigateDown";
        
        public Actions(String name) {
            super(name);
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            String name = this.getName();
            BasicToolBarUI basicToolBarUI = (BasicToolBarUI)BasicLookAndFeel.getUIOfType(((JToolBar)actionEvent.getSource()).getUI(), BasicToolBarUI.class);
            if ("navigateRight" == name) {
                basicToolBarUI.navigateFocusedComp(3);
            }
            else if ("navigateLeft" == name) {
                basicToolBarUI.navigateFocusedComp(7);
            }
            else if ("navigateUp" == name) {
                basicToolBarUI.navigateFocusedComp(1);
            }
            else if ("navigateDown" == name) {
                basicToolBarUI.navigateFocusedComp(5);
            }
        }
    }
    
    private class Handler implements ContainerListener, FocusListener, MouseInputListener, PropertyChangeListener
    {
        JToolBar tb;
        boolean isDragging;
        Point origin;
        
        private Handler() {
            this.isDragging = false;
            this.origin = null;
        }
        
        @Override
        public void componentAdded(ContainerEvent containerEvent) {
            Component child = containerEvent.getChild();
            if (BasicToolBarUI.this.toolBarFocusListener != null) {
                child.addFocusListener(BasicToolBarUI.this.toolBarFocusListener);
            }
            if (BasicToolBarUI.this.isRolloverBorders()) {
                BasicToolBarUI.this.setBorderToRollover(child);
            }
            else {
                BasicToolBarUI.this.setBorderToNonRollover(child);
            }
        }
        
        @Override
        public void componentRemoved(ContainerEvent containerEvent) {
            Component child = containerEvent.getChild();
            if (BasicToolBarUI.this.toolBarFocusListener != null) {
                child.removeFocusListener(BasicToolBarUI.this.toolBarFocusListener);
            }
            BasicToolBarUI.this.setBorderToNormal(child);
        }
        
        @Override
        public void focusGained(FocusEvent focusEvent) {
            BasicToolBarUI.this.focusedCompIndex = BasicToolBarUI.this.toolBar.getComponentIndex(focusEvent.getComponent());
        }
        
        @Override
        public void focusLost(FocusEvent focusEvent) {
        }
        
        @Override
        public void mousePressed(MouseEvent mouseEvent) {
            if (!this.tb.isEnabled()) {
                return;
            }
            this.isDragging = false;
            return;
        }
        
        @Override
        public void mouseReleased(MouseEvent mouseEvent) {
            if (!this.tb.isEnabled()) {
                return;
            }
            if (this.isDragging) {
                Point point = mouseEvent.getPoint();
                if (this.origin == null) {
                    this.origin = mouseEvent.getComponent().getLocationOnScreen();
                }
                BasicToolBarUI.this.floatAt(point, this.origin);
            }
            this.origin = null;
            this.isDragging = false;
            return;
        }
        
        @Override
        public void mouseDragged(MouseEvent mouseEvent) {
            if (!this.tb.isEnabled()) {
                return;
            }
            this.isDragging = true;
            Point point = mouseEvent.getPoint();
            if (this.origin == null) {
                this.origin = mouseEvent.getComponent().getLocationOnScreen();
            }
            BasicToolBarUI.this.dragTo(point, this.origin);
            return;
        }
        
        @Override
        public void mouseClicked(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseEntered(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseExited(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseMoved(MouseEvent mouseEvent) {
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            String propertyName = propertyChangeEvent.getPropertyName();
            if (propertyName == "lookAndFeel") {
                BasicToolBarUI.this.toolBar.updateUI();
            }
            else if (propertyName == "orientation") {
                Component[] components = BasicToolBarUI.this.toolBar.getComponents();
                int intValue = (int)propertyChangeEvent.getNewValue();
                for (int i = 0; i < components.length; ++i) {
                    if ((components[i] instanceof JToolBar.Separator)) {
                        JToolBar.Separator separator = (JToolBar.Separator)components[i];
                        if (intValue == 0) {
                            separator.setOrientation(1);
                        }
                        else {
                            separator.setOrientation(0);
                        }
                        Dimension separatorSize = separator.getSeparatorSize();
                        if (separatorSize != null && separatorSize.width != separatorSize.height) {
                            separator.setSeparatorSize(new Dimension(separatorSize.height, separatorSize.width));
                        }
                    }
                }
            }
            else if (propertyName == BasicToolBarUI.IS_ROLLOVER) {
                BasicToolBarUI.this.installNormalBorders(BasicToolBarUI.this.toolBar);
                BasicToolBarUI.this.setRolloverBorders((boolean)propertyChangeEvent.getNewValue());
            }
        }
    }
    
    protected class FrameListener extends WindowAdapter
    {
        @Override
        public void windowClosing(WindowEvent windowEvent) {
            if (BasicToolBarUI.this.toolBar.isFloatable()) {
                if (BasicToolBarUI.this.dragWindow != null) {
                    BasicToolBarUI.this.dragWindow.setVisible(false);
                }
                BasicToolBarUI.this.floating = false;
                if (BasicToolBarUI.this.floatingToolBar == null) {
                    BasicToolBarUI.this.floatingToolBar = BasicToolBarUI.this.createFloatingWindow(BasicToolBarUI.this.toolBar);
                }
                if ((BasicToolBarUI.this.floatingToolBar instanceof Window)) {
                    ((Window)BasicToolBarUI.this.floatingToolBar).setVisible(false);
                }
                BasicToolBarUI.this.floatingToolBar.getContentPane().remove(BasicToolBarUI.this.toolBar);
                String constraintBeforeFloating = BasicToolBarUI.this.constraintBeforeFloating;
                if (BasicToolBarUI.this.toolBar.getOrientation() == 0) {
                    if (constraintBeforeFloating == "West" || constraintBeforeFloating == "East") {
                        constraintBeforeFloating = "North";
                    }
                }
                else if (constraintBeforeFloating == "North" || constraintBeforeFloating == "South") {
                    constraintBeforeFloating = "West";
                }
                if (BasicToolBarUI.this.dockingSource == null) {
                    BasicToolBarUI.this.dockingSource = BasicToolBarUI.this.toolBar.getParent();
                }
                if (BasicToolBarUI.this.propertyListener != null) {
                    UIManager.removePropertyChangeListener(BasicToolBarUI.this.propertyListener);
                }
                BasicToolBarUI.this.dockingSource.add(BasicToolBarUI.this.toolBar, constraintBeforeFloating);
                BasicToolBarUI.this.dockingSource.invalidate();
                Container parent = BasicToolBarUI.this.dockingSource.getParent();
                if (parent != null) {
                    parent.validate();
                }
                BasicToolBarUI.this.dockingSource.repaint();
            }
        }
    }
    
    protected class ToolBarContListener implements ContainerListener
    {
        @Override
        public void componentAdded(ContainerEvent containerEvent) {
            BasicToolBarUI.access$500(BasicToolBarUI.this).componentAdded(containerEvent);
        }
        
        @Override
        public void componentRemoved(ContainerEvent containerEvent) {
            BasicToolBarUI.access$500(BasicToolBarUI.this).componentRemoved(containerEvent);
        }
    }
    
    protected class ToolBarFocusListener implements FocusListener
    {
        @Override
        public void focusGained(FocusEvent focusEvent) {
            BasicToolBarUI.access$500(BasicToolBarUI.this).focusGained(focusEvent);
        }
        
        @Override
        public void focusLost(FocusEvent focusEvent) {
            BasicToolBarUI.access$500(BasicToolBarUI.this).focusLost(focusEvent);
        }
    }
    
    protected class PropertyListener implements PropertyChangeListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            BasicToolBarUI.access$500(BasicToolBarUI.this).propertyChange(propertyChangeEvent);
        }
    }
    
    public class DockingListener implements MouseInputListener
    {
        protected JToolBar toolBar;
        protected boolean isDragging;
        protected Point origin;
        
        public DockingListener(JToolBar toolBar) {
            this.isDragging = false;
            this.origin = null;
            this.toolBar = toolBar;
            BasicToolBarUI.access$500(BasicToolBarUI.this).tb = toolBar;
        }
        
        @Override
        public void mouseClicked(MouseEvent mouseEvent) {
            BasicToolBarUI.access$500(BasicToolBarUI.this).mouseClicked(mouseEvent);
        }
        
        @Override
        public void mousePressed(MouseEvent mouseEvent) {
            BasicToolBarUI.access$500(BasicToolBarUI.this).tb = this.toolBar;
            BasicToolBarUI.access$500(BasicToolBarUI.this).mousePressed(mouseEvent);
            this.isDragging = BasicToolBarUI.access$500(BasicToolBarUI.this).isDragging;
        }
        
        @Override
        public void mouseReleased(MouseEvent mouseEvent) {
            BasicToolBarUI.access$500(BasicToolBarUI.this).tb = this.toolBar;
            BasicToolBarUI.access$500(BasicToolBarUI.this).isDragging = this.isDragging;
            BasicToolBarUI.access$500(BasicToolBarUI.this).origin = this.origin;
            BasicToolBarUI.access$500(BasicToolBarUI.this).mouseReleased(mouseEvent);
            this.isDragging = BasicToolBarUI.access$500(BasicToolBarUI.this).isDragging;
            this.origin = BasicToolBarUI.access$500(BasicToolBarUI.this).origin;
        }
        
        @Override
        public void mouseEntered(MouseEvent mouseEvent) {
            BasicToolBarUI.access$500(BasicToolBarUI.this).mouseEntered(mouseEvent);
        }
        
        @Override
        public void mouseExited(MouseEvent mouseEvent) {
            BasicToolBarUI.access$500(BasicToolBarUI.this).mouseExited(mouseEvent);
        }
        
        @Override
        public void mouseDragged(MouseEvent mouseEvent) {
            BasicToolBarUI.access$500(BasicToolBarUI.this).tb = this.toolBar;
            BasicToolBarUI.access$500(BasicToolBarUI.this).origin = this.origin;
            BasicToolBarUI.access$500(BasicToolBarUI.this).mouseDragged(mouseEvent);
            this.isDragging = BasicToolBarUI.access$500(BasicToolBarUI.this).isDragging;
            this.origin = BasicToolBarUI.access$500(BasicToolBarUI.this).origin;
        }
        
        @Override
        public void mouseMoved(MouseEvent mouseEvent) {
            BasicToolBarUI.access$500(BasicToolBarUI.this).mouseMoved(mouseEvent);
        }
    }
    
    protected class DragWindow extends Window
    {
        Color borderColor;
        int orientation;
        Point offset;
        
        DragWindow(Window owner) {
            super(owner);
            this.borderColor = Color.gray;
            this.orientation = BasicToolBarUI.this.toolBar.getOrientation();
        }
        
        public int getOrientation() {
            return this.orientation;
        }
        
        public void setOrientation(int orientation) {
            if (this.isShowing()) {
                if (orientation == this.orientation) {
                    return;
                }
                this.orientation = orientation;
                Dimension size = this.getSize();
                this.setSize(new Dimension(size.height, size.width));
                if (this.offset != null) {
                    if (BasicGraphicsUtils.isLeftToRight(BasicToolBarUI.this.toolBar)) {
                        this.setOffset(new Point(this.offset.y, this.offset.x));
                    }
                    else if (orientation == 0) {
                        this.setOffset(new Point(size.height - this.offset.y, this.offset.x));
                    }
                    else {
                        this.setOffset(new Point(this.offset.y, size.width - this.offset.x));
                    }
                }
                this.repaint();
            }
        }
        
        public Point getOffset() {
            return this.offset;
        }
        
        public void setOffset(Point offset) {
            this.offset = offset;
        }
        
        public void setBorderColor(Color borderColor) {
            if (this.borderColor == borderColor) {
                return;
            }
            this.borderColor = borderColor;
            this.repaint();
            return;
        }
        
        public Color getBorderColor() {
            return this.borderColor;
        }
        
        @Override
        public void paint(Graphics g) {
            BasicToolBarUI.this.paintDragWindow(g);
            super.paint(g);
        }
        
        @Override
        public Insets getInsets() {
            return new Insets(1, 1, 1, 1);
        }
    }
}
